# ~ encoding: utf-8 ~
import unittest
import datetime

import conf

from utils import get_shell_variable

SERVER_CONFIG_FILE_TEMPLATE = '/etc/opt/fierro%s/fierroserver.conf'
SENDOPERATIONS_CONFIG_FILE_TEMPLATE = '/etc/opt/fierroutils/fierroLocalRemoto%s.conf'


class TestOfflineSaleBillCreation(unittest.TestCase):

    SALEBILL_DESCRIPTION = 'FC de prueba de instalacion'
    SALEBILL_NUMBER = 98989898
    SALEBILL_AMOUNT = 1.00

    IVA_ALIQUOT_EXEMPT_ID = 1

    def __init__(self, methodName, fierro_instance=''):
        super(TestOfflineSaleBillCreation, self).__init__(methodName)

        self.fierro_instance = fierro_instance
        self.test_method_name = methodName
        self.server_config_file = SERVER_CONFIG_FILE_TEMPLATE % fierro_instance
        self.sendoperations_config_file = SENDOPERATIONS_CONFIG_FILE_TEMPLATE % fierro_instance

    def shortDescription(self):
        if self.fierro_instance:
            return '%s (instancia %s) (%s)' % (self.test_method_name,
                                               self.fierro_instance,
                                               '.'.join(self.id().split('.')[:-1]))

    def setUp(self):
        config = {}
        execfile(self.server_config_file, config)
        self.LOCAL_SERVER_PORT = config['SERVER_PORT']

        self.FIERRO_USER = get_shell_variable(self.sendoperations_config_file, 'FIERRO_USER')
        self.FIERRO_PASS = get_shell_variable(self.sendoperations_config_file, 'FIERRO_PASS')
        self.FIERRO_SUC = get_shell_variable(self.sendoperations_config_file, 'FIERRO_SUC')

        self.QUEUE_SERVER = get_shell_variable(self.sendoperations_config_file, 'QUEUE_SERVER')
        self.QUEUE_USER = get_shell_variable(self.sendoperations_config_file, 'QUEUE_USER')
        self.QUEUE_PASS = get_shell_variable(self.sendoperations_config_file, 'QUEUE_PASS')
        self.ONLINE_SERVER = get_shell_variable(self.sendoperations_config_file, 'FIERRO_SERVER')

    def test_crear_factura_offline_local(self):
        from colofon.client.cli.fierroCLILauncher import login
        login('PYROLOC://localhost:%s' % self.LOCAL_SERVER_PORT,
                self.FIERRO_USER,
                self.FIERRO_PASS,
                self.FIERRO_SUC)

        from colofon.client.model import session
        from colofon.client.model.sale import SaleBill, SaleBillIva
        from colofon.client.model.paramManager import getParam
        from colofon.client.model.payment import CashPayment
        from colofon.client.model.managers import IvaAliquotManager, SaleBillManager, LoginManager

        sale_bill_id = None

        def cleanUp():
            from colofon.client.ipc import initManagers
            if sale_bill_id:
                SaleBillManager.cancelItem(sale_bill_id)
                SaleBillManager.removeItem(sale_bill_id)
            LoginManager.logout(session.currentSession)
            initManagers.uninitManagers()

        self.addCleanup(cleanUp)

        sucursal = session.currentSession.sucursal
        sale_point = sucursal.defaultSalePoint
        client = getParam('SaleBillClient')

        sale_bill = SaleBill.newObject(sucursal=sucursal,
                                       salePoint=sale_point,
                                       date=datetime.date.today(),
                                       dueDate=datetime.date.today(),
                                       client=client,
                                       docType=SaleBill.TYPE_SALE_BILL_GENERAL,
                                       type=getParam('SaleBillTypeMinorista'),
                                       concepts=[],
                                       ivaAmounts=[],
                                       amount=self.SALEBILL_AMOUNT,
                                       currency=getParam('Currency'),
                                       salesman=None,
                                       number=self.SALEBILL_NUMBER,
                                       printTransports=[])
        sale_bill.description = self.SALEBILL_DESCRIPTION

        if getParam("IvaEnabled"):
            iva_aliquot = IvaAliquotManager.findByPrimaryKey(self.IVA_ALIQUOT_EXEMPT_ID)
            iva_amounts = [SaleBillIva.newObject(ivaAliquot=iva_aliquot,
                                                 grossAmount=self.SALEBILL_AMOUNT,
                                                 percentage=0,
                                                 amount=0)]
            sale_bill.ivaAmounts = iva_amounts

        sale_bill.payments.append(CashPayment.newObject(amount=sale_bill.amount, currency=sale_bill.currency,
                                                        cashBox=sucursal.defaultCashBox))

        sale_bill_id = SaleBillManager.addItem(sale_bill)

        new_sale_bill = SaleBillManager.findByPrimaryKey(sale_bill_id)

        self.assertEquals(sale_bill.number, new_sale_bill.number)

    def test_enviar_operacion_offline_a_online(self):
        from colofon.client.cli.fierroCLILauncher import login
        login(self.ONLINE_SERVER, self.FIERRO_USER, self.FIERRO_PASS, self.FIERRO_SUC)
        from colofon.client.model import session
        from colofon.client.model.managers import SaleBillManager, LoginManager
        from colofon.client.model.saleFilters import SaleBillNumberLikeEndFilter
        from colofon.client.model.MessageDispatcher import MessageDispatcher

        for msg in self.queue_iterator():
            if self.is_test_sale_bill(msg):
                MessageDispatcher.dispatch(msg, params={})
                break

        online_bill = SaleBillManager.queryItems(SaleBillNumberLikeEndFilter(str(self.SALEBILL_NUMBER)))
        self.assertEquals(len(online_bill), 1)
        online_bill = online_bill[0]

        def cleanUp():
            from colofon.client.ipc import initManagers
            SaleBillManager.cancelItem(online_bill.code)
            SaleBillManager.removeItem(online_bill.code)
            LoginManager.logout(session.currentSession)
            initManagers.uninitManagers()
        self.addCleanup(cleanUp)

        self.assertEquals(online_bill.number, self.SALEBILL_NUMBER)
        self.assertEquals(online_bill.amount, self.SALEBILL_AMOUNT)
        self.assertEquals(online_bill.description, self.SALEBILL_DESCRIPTION)

    def is_test_sale_bill(self, msg):
        type_ = msg.get('type')
        number = msg.get('object', {}).get('number')
        return type_ == 'saleBillAdded' and number == self.SALEBILL_NUMBER

    def queue_iterator(self):
        from fierrotools.messageQueue import FSMessageQueue
        queue = FSMessageQueue(self.QUEUE_SERVER)

        msg = None
        seen_messages = []
        while True:
            msg = queue.pop()

            if msg is None:
                break
            if not msg:
                continue

            # Si llegamos a un mensaje ya visto, significa que se terminó la cola
            if msg['uuid'] in seen_messages:
                queue.push(msg)
                break

            seen_messages.append(msg['uuid'])

            yield msg

            queue.push(msg)

    def test_no_quedan_operaciones_de_prueba_en_queueserver(self):
        for msg in self.queue_iterator():
            self.assertFalse(self.is_test_sale_bill(msg))


def load_tests(loader, tests, pattern):
    suite = unittest.TestSuite()
    if 'offline' not in conf.TEST_COMPONENTS:
        return suite

    test_cases = loader.getTestCaseNames(TestOfflineSaleBillCreation)

    for instance in conf.FIERROSERVER_INSTANCES:
        for test_case in test_cases:
            suite.addTest(TestOfflineSaleBillCreation(test_case, fierro_instance=instance))

    return suite
